# Демо-программа для определения i2c-преобразователя.
# Алгоритм демо-программы: сначала пытаемся подключиться к контроллеру ch341, если не удается, то подключаемся к cp2112
import sys
from ctypes import *
from ctypes.util import find_library
import platform
import os

# Подключаем внешнюю библиотеку для работы с SDK
libName = ""
platform = platform.system()
if platform == "Windows":
    libName = "librisdk.dll"
if platform == "Linux":
    libName = "librisdk.so"

libPath = os.getenv("LIB_RISDK")
print(libPath)

lib = cdll.LoadLibrary(libPath + libName)


# Указываем типы аргументов для функций библиотеки RI_SDK
lib.RI_SDK_InitSDK.argtypes = [c_int, c_char_p]
lib.RI_SDK_CreateModelComponent.argtypes = [c_char_p, c_char_p, c_char_p, POINTER(c_int), c_char_p]
lib.RI_SDK_LinkPWMToController.argtypes = [c_int, c_int, c_uint8, c_char_p]

# Текст ошибки подключения к шине
failedConnectAnyBusError = "Failed to connect on any bus"


def main():
    errTextC = create_string_buffer(1000)  # Текст ошибки. C type: char*
    i2c = c_int()
    pwm = c_int()

    # Инициализация библиотеки RI SDK с уровнем логирования 3
    errCode = lib.RI_SDK_InitSDK(3, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

        # Создание компонента ШИМ модели pca9685
    errCode = lib.RI_SDK_CreateModelComponent("connector".encode(), "pwm".encode(), "pca9685".encode(), pwm, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Создание компонента i2c адаптера модели ch341
    errCode = lib.RI_SDK_CreateModelComponent("connector".encode(), "i2c_adapter".encode(), "ch341".encode(), i2c,
                                              errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Связывание i2c с ШИМ
    errCode = lib.RI_SDK_LinkPWMToController(pwm, i2c, 0x40, errTextC)
    if errCode != 0:
        if failedConnectAnyBusError in errTextC.raw.decode():
            # Создание компонента i2c адаптера модели cp2112
            errCode = lib.RI_SDK_CreateModelComponent("connector".encode(), "i2c_adapter".encode(), "cp2112".encode(),
                                                      i2c, errTextC)
            if errCode != 0:
                print(errCode, errTextC.raw.decode())
                sys.exit(2)

            # Связывание i2c с ШИМ
            errCode = lib.RI_SDK_LinkPWMToController(pwm, i2c, 0x40, errTextC)
            if errCode != 0:
                print(errCode, errTextC.raw.decode())
                sys.exit(2)
            print("Ваш i2c-преобразователь: cp2112")
            return

        print(errCode, errTextC.raw.decode())
        sys.exit(2)
    print("Ваш i2c-преобразователь: ch341")


main()
